#<profile4xi profile = "03" />
#<program4xi program = "dupfd4xi.py">
#<abstract4xi abstract = "providing class DupFd4Xi"/>
#<refer4xi reference = "reduction.py"/>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'DupFd4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                    # system ones
    import os
    import sys
    import signal
except:
    pass
try:                    # threading ones       
    import threading                
    import _thread   
except:
    pass
try:                    # winapi
    import _winapi
    from _winapi import CloseHandle
    from _winapi import DuplicateHandle
    from _winapi import DUPLICATE_SAME_ACCESS
    from _winapi import CreateProcess
    from _winapi import GetCurrentProcess
    from _winapi import GetExitCodeProcess
    from _winapi import INFINITE
    from _winapi import TerminateProcess
    from _winapi import WaitForSingleObject
    from _winapi import WAIT_OBJECT_0
    from _winapi import WAIT_TIMEOUT    
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
    
# the following fails !
# try:                             # Python ones
#     from multiprocessing.reduction import DupFd
# except:
#     print("dupfd4xi.py: *** failed in from multiprocessing.reduction import DupFd ***")
   
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

from examine4xi import examine4xi

from resourcesharer4xi import ResourceSharer4Xi

from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi   

#"
#/>

# =================================================================================================
#<class4xi class = "DupFd4Xi" base = "object" child = "None" synopsis = "Duplicates file descriptor" usage = "
#   reduction.py
#       reduce_socket(s):
#           df = DupFd(s.fileno())
#           return rebuild_socket, (df, s.family, s.type, s.proto)   
#   reduce_socket4xi.py
#       df2xi = DupFd4Xi(s.fileno())
#       return rebuild_socket4xi, (df2xi, s.family, s.type, s.proto) 
#
#>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class DupFd4Xi(object):  
    '''
    Duplicates file descriptor in non-Windows environment
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('detach', 'sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    resource_sharer2xi = ResourceSharer4Xi()
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DupFd4Xi" usage = "initializes DupFd4Xi with (self, fd)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, fd):
        '''
        initializes DupFd4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupFd4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/4a) started" + "; init-of-DupFd4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------    
        new_fd2xi = os.dup(fd)
        
        message2print = "{(Step 4/4b) set" + "; new_fd2xi-set-by-init-of-DupFd4Xi to: " + str(new_fd2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #  
        
            
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-DupFd-init-performing-by-duplicate4xi: " + str(DupFd__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            DupFd.__init__(self, fd)
            
            message2print = "{(StepOriginal) performed" + "; Original-DupFd-init-performed-by-duplicate4xi: " + str(DupFd__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        else:                
            
            # -------------------------------------------------------------------------------------------------
            #<function4xi function = "send" usage = "
            #    self._id = resource_sharer2xi.register(send, close)
            #"> 
            #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
            # -------------------------------------------------------------------------------------------------
            def send(conn, pid):
                send_handle(conn, new_fd2xi, pid) 
            #</function4xi>
            
            # -------------------------------------------------------------------------------------------------
            #<function4xi function = "close" usage = "
            #    self._id = resource_sharer2xi.register(send, close)
            #"> 
            #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
            # -------------------------------------------------------------------------------------------------        
            def close():
                os.close(new_fd2xi)
            #</function4xi>
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/4)            
            # -----------------------------------------------------------------------------------------    
            self._id = resource_sharer2xi.register(send, close)
            message2print = "{(Step 3/4) set" + "; id-of-dupfd2xi-set-by-init-of-DupFd4Xi to: " + str(self._id)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #  
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4) 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 4/4a) activated" + "; dupfd2xi-of-DupFd4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #                 
            
            message2print = "{(Step 4/4b) inherited" + "; object-inherited-by-dupfd2xi-of-DupFd4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #    
                     
            message2print = "{(Step 4/4c) returning" + "; init-of-DupFd4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #                        
                    
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DupFd4Xi" usage = "detaches DupFd4Xi with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def detach(self):
        '''
        detaches DupFd4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupFd4Xi"
        method2print = "detach"
        
        message2print = "{(Step 1/4) started" + "; detach-of-DupFd4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
            
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-DupFd-detach-performing-by-duplicate4xi: " + str(DupFd.detach) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            return DupFd.detach(self)
            

        else:                
                    
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/4)            
            # -----------------------------------------------------------------------------------------    
            conn2xi = resource_sharer2xi.get_connection(self._id)
            message2print = "{(Step 2/4a) set" + "; conn2xi-set-by-detach-of-DupFd4Xi to: " + str(conn2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #  
            
            try:
                message2print = "{(Step 2/4b) passing" + "; conn2xi-passing-to-recv_handle-by-detach-of-DupFd4Xi: " + str(conn2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                #
                  
                message2print = "{(Step 2/4c) returning" + "; recv_handle-returning-by-detach-of-DupFd4Xi: " + str(recv_handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                #
                
                return recv_handle(conn2xi)
            finally:
                message2print = "{(Step 2/4d) closing" + "; connection-closing-by-detach-of-DupFd4Xi: " + str(conn2xi.close)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                #   
            
                conn2xi.close()
            
                # -----------------------------------------------------------------------------------------
                # (Step 3/4)            
                # -----------------------------------------------------------------------------------------    
                pass        
                            
                # -----------------------------------------------------------------------------------------    
                # (Step 4/4) 
                # -----------------------------------------------------------------------------------------                 
                message2print = "{(Step 4/4) returning" + "; detach-of-DupFd4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                #                        
                
    #</methodize4xi>    
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of dupfd4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

